//
//  AppDelegate.swift
//  VMS
//
//  Copyright (c) 2015 Skymotion. All rights reserved.
//

import UIKit

let kNotifKey = "key"
let kNotifKey_IncomingCall = "icall"
let kNotifKey_IncomingMsg = "imsg"
let kNotifCall_SessionId = "sid"
let kNotifCall_RemoteInfo = "rinfo"
let kNotifKey_IncomingIm = "iim"

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {
    
    static let AcceptCallAction = "AcceptCallAction"
    static let RejectCallAction = "RejectCallAction"
    static let CallNotificationCategory = "CallNotificationCategory"

    var window: UIWindow?
    var splash: UIView?
    var lastCallNotification: UILocalNotification?
    var checkIconLaunch: Bool = false

    func application(application: UIApplication, didFinishLaunchingWithOptions launchOptions: [NSObject: AnyObject]?) -> Bool {

        setupLog()
        setupAppearance()

        DDLogVerbose("Hello start: \(application.applicationState)")

        NSNotificationCenter.defaultCenter().addObserver(self, selector: "createNotificationChecker:", name: UIApplicationWillEnterForegroundNotification, object: nil)
        NSNotificationCenter.defaultCenter().addObserver(self, selector: "createNotificationChecker:", name: UIApplicationDidFinishLaunchingNotification, object: nil)
        NSNotificationCenter.defaultCenter().addObserver(self, selector: "onCallNotification:", name: NOTIFICATION_CALL_EVENT, object: nil)
        
        Phone.sharedInstance.start()
        
        let tabs: UITabBarController = window?.rootViewController as! UITabBarController
        tabs.selectedIndex = 2
        
        if let views = NSBundle.mainBundle().loadNibNamed("LaunchScreen", owner: nil, options: nil) {
            if views.count > 0 {
                let spl = views[0] as! UIView
                spl.translatesAutoresizingMaskIntoConstraints = false
                tabs.view.addSubview(spl)
                let dict = [ "splash": spl ]
                let hconstraints = NSLayoutConstraint.constraintsWithVisualFormat("H:|[splash]|", options: [], metrics: nil, views: dict)
                tabs.view.addConstraints(hconstraints)
                let vconstraints = NSLayoutConstraint.constraintsWithVisualFormat("V:|[splash]|", options: [], metrics: nil, views: dict)
                tabs.view.addConstraints(vconstraints)
                splash = spl
            }
        }
        return true
    }

    func applicationWillResignActive(application: UIApplication) {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
    }

    func applicationDidEnterBackground(application: UIApplication) {
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
        DDLogVerbose("App did enter background");
    }

    func applicationWillEnterForeground(application: UIApplication) {
        // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
        
        UIApplication.sharedApplication().applicationIconBadgeNumber = 0
    }

    func applicationDidBecomeActive(application: UIApplication) {
        // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
        
        if splash != nil && !Phone.sharedInstance.configured {
            performSelector("displaySettings", withObject: nil, afterDelay: 0.1)
        } else {
            removeSplash()
        }
        
        if (checkIconLaunch) {
            checkIconLaunch = false
            if  let notif = lastCallNotification {
                let data = notif.userInfo as! [String: AnyObject]
                NSNotificationCenter.defaultCenter().postNotificationName(NOTIFICATION_CALL_EVENT, object: NSNumber(integer: CallEvents.Incoming.rawValue), userInfo: [
                    CALL_ID_ARGUMENT: data[kNotifCall_SessionId]!,
                    CONTACT_ARGUMENT: data[kNotifCall_RemoteInfo]!
                ])
                lastCallNotification = nil
            }
        }
    }

    func applicationWillTerminate(application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    }
    
    func application(application: UIApplication, didReceiveLocalNotification notification: UILocalNotification) {
        if let notifKey: String = (notification.userInfo as! [ String: AnyObject ])[kNotifKey] as? String {
        
            if notifKey == kNotifKey_IncomingCall {
                application.applicationIconBadgeNumber -= notification.applicationIconBadgeNumber
            } else if notifKey == kNotifKey_IncomingIm {
                application.applicationIconBadgeNumber -= notification.applicationIconBadgeNumber
                NSNotificationCenter.defaultCenter().postNotificationName(NOTIFICATION_NEW_IM, object: nil)
            }
        }
    }
    
    func application(application: UIApplication, handleActionWithIdentifier identifier: String?, forLocalNotification notification: UILocalNotification, completionHandler: () -> Void) {
        let data = notification.userInfo as! [String: AnyObject]
        if let notifKey: String = data[kNotifKey] as? String {
            
            if notifKey == kNotifKey_IncomingCall {
                let callid = data[kNotifCall_SessionId] as! NSNumber
                if identifier == AppDelegate.AcceptCallAction {
                    let sb = UIStoryboard(name: "Call", bundle: nil)
                    if let vc: CallViewController = sb.instantiateViewControllerWithIdentifier("CallViewController") as? CallViewController {
                        let cid = callid.integerValue
                        Phone.sharedInstance.answerCall(cid)
                        vc.incoming = true
                        vc.callId = cid
                        vc.number = data[kNotifCall_RemoteInfo] as! String
                        self.window?.rootViewController?.presentViewController(vc, animated: false, completion: {})
                    }
                } else if identifier == AppDelegate.RejectCallAction {
                    Phone.sharedInstance.hangUpCall(callid.integerValue)
                }
                checkIconLaunch = false
                lastCallNotification = nil
            }
        }
        completionHandler()
    }
    
    func setupLog() {
        DDLog.addLogger(DDASLLogger.sharedInstance())
        DDLog.addLogger(DDTTYLogger.sharedInstance())

#if DEBUG
        defaultDebugLevel = DDLogLevel.Verbose
#else
        defaultDebugLevel = DDLogLevel.Warning
#endif
    }
    
    func displaySettings() {
        let vc = UIStoryboard(name: "Discovery", bundle: nil).instantiateInitialViewController()

        window?.rootViewController?.presentViewController(vc!, animated: false, completion: {
            self.removeSplash()
        })
    }
    
    func removeSplash() {
        self.splash?.removeFromSuperview()
        self.splash = nil
        AddressBook.sharedInstance.reloadContacts()
        registerNotifications()
    }
    
    func registerNotifications() {
        let acceptAction = UIMutableUserNotificationAction()
        acceptAction.destructive = false
        acceptAction.authenticationRequired = true
        acceptAction.activationMode = .Foreground
        acceptAction.identifier = AppDelegate.AcceptCallAction
        acceptAction.title = NSLocalizedString("call.action.accept", comment: "")
        
        let rejectAction = UIMutableUserNotificationAction()
        rejectAction.destructive = true
        rejectAction.authenticationRequired = true
        rejectAction.activationMode = .Foreground
        rejectAction.identifier = AppDelegate.RejectCallAction
        rejectAction.title = NSLocalizedString("call.action.reject", comment: "")
        
        let callCategory = UIMutableUserNotificationCategory()
        callCategory.identifier = AppDelegate.CallNotificationCategory
        callCategory.setActions([ acceptAction, rejectAction ], forContext: .Minimal)
        callCategory.setActions([ acceptAction, rejectAction ], forContext: .Default)
        
        let settings = UIUserNotificationSettings(forTypes:
            [.Alert, .Badge, .Sound], categories: Set([ callCategory ]) )
        UIApplication.sharedApplication().registerUserNotificationSettings(settings)
    }
    
    func setupAppearance() {
        // TAB BAR
        let tabAppearance = UITabBar.appearance()
        tabAppearance.tintColor = Theme.Colors.TabBarTint
        /*
        UITabBarItem.appearance().setTitleTextAttributes([
            NSFontAttributeName: UIFont(name: "Cuprum-Regular", size: 10)!
            ], forState: UIControlState.Normal)
        */
        
        // NAV BAR
        let navAppearance = UINavigationBar.appearance()
        navAppearance.barTintColor = Theme.Colors.NavBarBackground
        navAppearance.translucent = false
        navAppearance.tintColor = Theme.Colors.NavBarText
        navAppearance.barStyle = .Black
        /*
        navAppearance.titleTextAttributes = [
            NSFontAttributeName: UIFont(name: "Cuprum-Regular", size: 18)!,
            NSForegroundColorAttributeName: Theme.Colors.NavBarText
        ]
        UIBarButtonItem.appearance().setTitleTextAttributes([
            NSFontAttributeName: UIFont(name: "Cuprum-Bold", size: 18)!,
            ], forState: UIControlState.Normal)
        
        // SEARCH BAR
        let searchFont = UIFont(name: "Cuprum-Regular", size: 16)!
        UITextField.VMS_appearanceWhenContainedIn(UISearchBar).defaultTextAttributes = [
            NSFontAttributeName: searchFont
        ]
        */
    }
    
    func listInstalledFonts() {
#if DEBUG
        DDLogVerbose("Installed fonts:")
        let families = UIFont.familyNames()
        for family in families {
            let fam = family 
            DDLogVerbose(fam)
            let names = UIFont.fontNamesForFamilyName(fam)
            for font in names {
                DDLogVerbose("\t" + font)
            }
        }
#endif
    }
    
    func createNotificationChecker(notification: NSNotification) {
        let launchOptions = notification.userInfo as! [String: AnyObject]?
        let localNotification: UILocalNotification?
    
    // This code will be called immediately after application:didFinishLaunchingWithOptions:.
        if let options = launchOptions {
            localNotification = options[UIApplicationLaunchOptionsLocalNotificationKey] as! UILocalNotification?
        } else {
            localNotification = nil
        }
    
        checkIconLaunch = !(localNotification != nil);
    }
    
    func onCallNotification(notification: NSNotification) {
        let status = (notification.object as! NSNumber).integerValue
        
        switch status {
        case CallEvents.Incoming.rawValue:
            let info = notification.userInfo as! [String : AnyObject]
            let callId = (info[CALL_ID_ARGUMENT] as! NSNumber).integerValue
            let contact = info[CONTACT_ARGUMENT] as! String
            
            if callId != kInvalidCallId {
                let app = UIApplication.sharedApplication()
                let phone = Phone.sharedInstance.phone
                let remotePartyNumber = phone.sipUriUsername(contact)

                AddressBook.sharedInstance.matchPhoneNumberAsync(remotePartyNumber, callback: { (pho: PhoneNumberCache?) -> Void in
                    let state = app.applicationState
                    
                    phone.setSpeakerphoneOn(true)
                    
                    if state == .Background || state == .Inactive {
                        var number: String = remotePartyNumber
                        if let name = pho?.contact?.displayFullName() {
                            number = name
                        }
                        let localNotification = UILocalNotification()
                        localNotification.alertBody = String(format: NSLocalizedString("call.state.incoming.notif", comment: ""), arguments: [number])
                        localNotification.applicationIconBadgeNumber = ++app.applicationIconBadgeNumber
                        localNotification.repeatInterval = NSCalendarUnit()
                        localNotification.category = AppDelegate.CallNotificationCategory
                        let userInfo = [
                            kNotifKey: kNotifKey_IncomingCall,
                            kNotifCall_SessionId: NSNumber(integer: callId),
                            kNotifCall_RemoteInfo: contact
                        ]
                        localNotification.userInfo = userInfo
                        app.presentLocalNotificationNow(localNotification)
                        self.lastCallNotification = localNotification
                    } else {
                        let sb = UIStoryboard(name: "Call", bundle: nil)
                        let nav: UINavigationController = sb.instantiateViewControllerWithIdentifier("IncomingCallNavigationController") as! UINavigationController
                        let vc: IncomingCallViewController = nav.viewControllers[0] as! IncomingCallViewController
                        vc.callId = callId
                        vc.number = contact
                        vc.incoming = true
                        self.window?.rootViewController?.presentViewController(nav, animated: false, completion: {})
                    }
                })
            }
            break;
        case CallEvents.Disconnected.rawValue:
            lastCallNotification = nil
            break;
        default:
            break;
        }
            
    }
}

