//
//  AbstractDialerViewController.swift
//  VMS
//
//  Copyright (c) 2015 Skymotion. All rights reserved.
//

import UIKit
import CustomViews
import AVFoundation

class AbstractDialerViewController: UIViewController, AVAudioPlayerDelegate {

    @IBOutlet weak var phoneInput: UITextField!
    @IBOutlet weak var button1: DialPadButton!
    @IBOutlet weak var button2: DialPadButton!
    @IBOutlet weak var button3: DialPadButton!
    @IBOutlet weak var button4: DialPadButton!
    @IBOutlet weak var button5: DialPadButton!
    @IBOutlet weak var button6: DialPadButton!
    @IBOutlet weak var button7: DialPadButton!
    @IBOutlet weak var button8: DialPadButton!
    @IBOutlet weak var button9: DialPadButton!
    @IBOutlet weak var button0: DialPadButton!
    @IBOutlet weak var buttonStar: DialPadButton!
    @IBOutlet weak var buttonHash: DialPadButton!
    
    var player: AVAudioPlayer? {
        willSet {
            player?.delegate = nil
            player?.stop()
        }
        
        didSet {
            player?.delegate = self
        }
    }
    
    @IBAction func clickNum(sender: DialPadButton) {
        if let num = sender.indentText {
            appendDigit(digit: num)
        }
    }
    
    @IBAction func clickStar(sender: DialPadButton) {
        appendDigit(digit: "*")
    }
    @IBAction func clickHash(sender: DialPadButton) {
        appendDigit(digit: "#")
    }
    
    func appendDigit(digit num: String) {
        playSoundForDigit(num)
    }
    
    func setupSoundPlayer(digit: String) -> AVAudioPlayer! {
        var suffix = digit
        switch (digit) {
        case "*":
            suffix = "star"
            break;
        case "#":
            suffix = "pound"
            break;
        default:
            break;
        }
        
        let soundPath = NSBundle.mainBundle().pathForResource("dtmf-\(suffix)", ofType: "wav")
        let soundUrl = NSURL.fileURLWithPath(soundPath!)
        
        do {
            let pla = try AVAudioPlayer(contentsOfURL: soundUrl, fileTypeHint: AVFileTypeWAVE)
            pla.prepareToPlay()
            return pla
        } catch let error as NSError {
            DDLogError("Error creating player for digit \(digit): \(error), \(error.userInfo)")
        }
        return nil
    }
    
    func playSoundForDigit(digit: String) {
        player = setupSoundPlayer(digit)
        player?.play()
    }
    
    deinit {
        player = nil
    }
    
    // MARK: - AVAudioPlayerDelegate
    func audioPlayerDidFinishPlaying(player: AVAudioPlayer, successfully flag: Bool) {
        self.player = nil
    }
    
}
