//
//  AbstractCallViewController.swift
//  VMS
//
//  Copyright (c) 2015 Skymotion. All rights reserved.
//

import UIKit

class AbstractCallViewController: UIViewController {
    
    var number: String! = nil
    var callId: Int = kInvalidCallId
    var incoming = false
    var nameOrPhone: String?
    var numberType: String?

    @IBOutlet weak var mainText: UILabel!
    @IBOutlet weak var subText: UILabel!
    @IBOutlet weak var hangup: UIButton!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        NSNotificationCenter.defaultCenter().addObserver(self, selector: "onPhoneNotification:", name: NOTIFICATION_PHONE_EVENT, object: nil)
        
        mainText.text = nameOrPhone
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        NSNotificationCenter.defaultCenter().addObserver(self, selector: "onCallNotification:", name: NOTIFICATION_CALL_EVENT, object: nil)
        
        UIDevice.currentDevice().proximityMonitoringEnabled = true
        let phone = Phone.sharedInstance
        
        if nameOrPhone == nil {
            var num = number
            var sipNum: String? = nil
            if !num.hasPrefix("<") {
                if num.hasPrefix("sip:") {
                    sipNum = "<\(num)>"
                }
            } else {
                sipNum = num
            }
            
            if let sip = sipNum {
                num = phone.phone.sipUriUsername(sip)
            }
            mainText?.text = num
            AddressBook.sharedInstance.matchPhoneNumberAsync(num, callback: { (pho: PhoneNumberCache?) -> Void in
                if let name = pho?.contact?.displayFullName() {
                    self.nameOrPhone = name
                    self.numberType = pho!.displayLabel()
                    self.mainText.text = name
                    self.onMatchNumber(name, type: self.numberType!)
                }
            })
        }
    }
    
    override func viewWillDisappear(animated: Bool) {
        super.viewWillDisappear(animated)
        
        UIDevice.currentDevice().proximityMonitoringEnabled = false
    }
    
    override func preferredStatusBarStyle() -> UIStatusBarStyle {
        return .LightContent
    }
    
    func close() {
        let presenting: UIViewController?
        if let nav = navigationController {
            presenting = nav.presentingViewController
        } else {
            presenting = presentingViewController
        }
        presenting?.dismissViewControllerAnimated(false, completion: {})
    }
    
    @IBAction func clickHangup(sender: UIButton) {
        Phone.sharedInstance.hangUpCall(self.callId)
    }
    
    func onCallNotification(notification: NSNotification) {
        let status = (notification.object as! NSNumber).integerValue
        let phone = Phone.sharedInstance.phone
        let app = UIApplication.sharedApplication()
        
        switch status {
        case CallEvents.Connected.rawValue:
            phone.setSpeakerphoneOn(false)
            app.idleTimerDisabled = true
            break;
        case CallEvents.Disconnected.rawValue:
            app.idleTimerDisabled = false
            close()
            break;
        case CallEvents.Alerting.rawValue:
            phone.setSpeakerphoneOn(true)
            break;
        default:
            break;
        }
    }
    
    func onPhoneNotification(notification: NSNotification) {
        
    }
    
    func onMatchNumber(name: String, type: String) {
        
    }
    
    deinit {
        NSNotificationCenter.defaultCenter().removeObserver(self)
    }
}
