//
//  CallViewController.swift
//  VMS
//
//  Copyright (c) 2015 Skymotion. All rights reserved.
//

import UIKit

class CallViewController: AbstractCallViewController {
    
    @IBOutlet weak var mute: UIButton!
    @IBOutlet weak var dialer: UIButton!
    @IBOutlet weak var speaker: UIButton!
    
    var startDate: NSDate?
    var updateTimer: NSTimer?
    var durationFormatter: DurationFormatter = DurationFormatter()
    
    @IBAction func clickDialer(sender: UIButton) {
        if let sb = self.storyboard {
            let vc: InCallDialerViewController = sb.instantiateViewControllerWithIdentifier("InCallDialerViewController") as! InCallDialerViewController
            vc.modalTransitionStyle = UIModalTransitionStyle.FlipHorizontal
            vc.callId = callId
            presentViewController(vc, animated: true, completion: {})
        }
        
    }
    
    @IBAction func clickMute(sender: UIButton) {
        mute.selected = !mute.selected
        let phone = Phone.sharedInstance.phone
        phone.muteMicrophone(self.callId, on: mute.selected)
    }
    
    @IBAction func clickSpeaker(sender: UIButton) {
        speaker.selected = !speaker.selected
        let phone = Phone.sharedInstance.phone
        phone.setSpeakerphoneOn(speaker.selected)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        durationFormatter.shortFormat = true
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        if isBeingPresented() || isMovingToParentViewController() {
            if !incoming {
                if let label = numberType {
                    subText?.text = String(format: NSLocalizedString("call.state.calling.label", comment: ""), label)
                } else {
                    subText?.text = NSLocalizedString("call.state.calling", comment: "")
                }
                let phone = Phone.sharedInstance
                self.callId = phone.startCall(self.number)
            } else {
                startDisplayingTimer()
            }
        }
        if startDate != nil {
            playTimer()
        }
    }
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
        
        if self.callId == kInvalidCallId {
            close()
        }
    }
    
    override func viewWillDisappear(animated: Bool) {
        super.viewWillDisappear(animated)
        
        pauseTimer()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func onCallNotification(notification: NSNotification) {
        super.onCallNotification(notification)
        
        let status = (notification.object as! NSNumber).integerValue
        
        switch status {
        case CallEvents.Connected.rawValue:
            startDisplayingTimer()
            break;
        default:
            break;
        }
        
    }
    
    deinit {
       pauseTimer()
    }
    
    private func startDisplayingTimer() {
        startDate = NSDate()
        playTimer()
    }
    
    private func pauseTimer() {
        updateTimer?.invalidate()
    }
    
    private func playTimer() {
        updateTimer = NSTimer.scheduledTimerWithTimeInterval(1.0, target: self, selector: "updateTimerDisplay:", userInfo: nil, repeats: true)
        updateTimerDisplay(updateTimer!)
    }
    
    func updateTimerDisplay(timer: NSTimer) {
        let disp = durationFormatter.stringFromNumber(NSNumber(double: NSDate().timeIntervalSinceDate(startDate!)))
        
        subText.text = disp
    }
}
