//
//  DialerViewController.swift
//  VMS
//
//  Copyright (c) 2015 Skymotion. All rights reserved.
//

import UIKit
import CustomViews
import AddressBook
import AddressBookUI
import ContactsUI

class DialerViewController: AbstractDialerViewController, ABNewPersonViewControllerDelegate, CNContactViewControllerDelegate {
    
    @IBOutlet weak var addPhoneNumber: UIButton!
    @IBOutlet weak var backspace: UIButton!
    @IBOutlet weak var callButton: UIButton!
    @IBOutlet weak var contactName: UILabel!
    
    static let PHONE_NUMBER_MATCH_DELAY = 0.2
    
    private var phoneFormatter: NBAsYouTypeFormatter!
    private var timer: NSTimer?
    
    private var matchedName: String? = nil
    private var matchedContactPhoneType: String? = nil
    
    private var matchedNameAttributed: NSAttributedString? {
        get {
            return contactName.attributedText
        }
        
        set {
            contactName.attributedText = newValue
            let num = phoneInput.text
            addPhoneNumber.hidden = num == nil || num!.isEmpty || newValue != nil
        }
    }
    
    @IBAction func clickCall(sender: UIButton) {

        if let number = phoneInput.text {
            if !number.isEmpty {
                startCall(number, displayType: matchedContactPhoneType, displayName: matchedName, completion: {
                    self.phoneInput.text = nil
                    self.backspace.hidden = true
                    self.addPhoneNumber.hidden = true
                    self.phoneFormatter.clear()
                })
                return
            }
        }
        
        loadLastCalledNumber()
    }
    
    @IBAction func clickAddPhoneNumber(sender: UIButton) {
        if #available(iOS 9.0, *) {
            addNumberToContacts9(phoneInput.text)
        } else {
            addNumberToContacts(phoneInput.text)
        }
    }
    
    @IBAction func clickBackspace(sender: UIButton) {
        self.matchedNameAttributed = nil
        self.matchedName = nil
        self.matchedContactPhoneType = nil
        let fullNum = phoneFormatter.removeLastDigit()
        phoneInput.text = fullNum
        if(fullNum.isEmpty) {
            backspace.hidden = true
            addPhoneNumber.hidden = true
        }
    }
    
    func clickPlus(sender: UILongPressGestureRecognizer) {
        if sender.state == .Began {
            appendDigit(digit: "+")
        }
    }
    
    override func appendDigit(digit num: String) {
        super.appendDigit(digit: num)
        
        timer?.invalidate()
        let tm = NSTimer(timeInterval: DialerViewController.PHONE_NUMBER_MATCH_DELAY, target: self, selector: "doMatchPhoneNumber", userInfo: nil, repeats: false)
        timer = tm
        NSRunLoop.mainRunLoop().addTimer(tm, forMode: NSRunLoopCommonModes)
        
        let fullNum = phoneFormatter.inputDigit(num)
        phoneInput.text = fullNum
        backspace.hidden = false
    }
    
    func loadLastCalledNumber() {
        if let num = CallLog.sharedInstance.lastCalledNumber() {
            let fullNum = phoneFormatter.inputString(num)
            phoneInput.text = fullNum
            backspace.hidden = false
            doMatchPhoneNumber()
        }
    }
    
    func doMatchPhoneNumber() {
        if let num = phoneInput.text {
            AddressBook.sharedInstance.matchPhoneNumberAsync(num, callback: { (pho: PhoneNumberCache?) -> Void in
                if let name = pho?.contact?.displayFullName() {
                    let numName = NSMutableAttributedString(string: name)
                    let label = pho!.displayLabel()
                    numName.appendAttributedString(NSAttributedString(string: " "))
                    let attr = [ NSFontAttributeName: self.contactName.font.VMS_boldFont() ]
                    numName.appendAttributedString(NSAttributedString(string: label, attributes: attr))
                    self.matchedNameAttributed = numName
                    self.matchedName = name
                    self.matchedContactPhoneType = label
                } else {
                    self.matchedNameAttributed = nil
                    self.matchedName = nil
                    self.matchedContactPhoneType = nil
                }
            })
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        NSNotificationCenter.defaultCenter().addObserver(self, selector: "addressBookDidChange:", name: AddressBook.DidChangeNotification, object: nil)
        
        phoneFormatter = NBAsYouTypeFormatter(regionCode: NSLocale.currentLocale().objectForKey(NSLocaleCountryCode) as! String?)
        backspace.hidden = true
        addPhoneNumber.hidden = true
        
        let longPress = UILongPressGestureRecognizer(target: self, action: "clickPlus:")
        button0.addGestureRecognizer(longPress)
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func addressBookDidChange(notification: NSNotification) {
        doMatchPhoneNumber()
    }
    
    deinit {
        NSNotificationCenter.defaultCenter().removeObserver(self)
    }
    
    // MARK: - ABNewPersonViewControllerDelegate
    func newPersonViewController(newPersonView: ABNewPersonViewController, didCompleteWithNewPerson person: ABRecord?) {
        newPersonView.dismissViewControllerAnimated(true, completion: {})
    }
    
    // MARK: - CNContactViewControllerDelegate
    @available(iOS 9.0, *)
    func contactViewController(viewController: CNContactViewController, shouldPerformDefaultActionForContactProperty property: CNContactProperty) -> Bool {
        return false
    }
    
    @available(iOS 9.0, *)
    func contactViewController(viewController: CNContactViewController, didCompleteWithContact contact: CNContact?) {
        dismissViewControllerAnimated(true, completion: {})
    }
}
