//
//  DiscoveryViewController.swift
//  VMS
//
//  Copyright © 2015 Skymotion. All rights reserved.
//

import UIKit
import Foundation

class DiscoveryViewController: UIViewController, UITableViewDelegate, UITableViewDataSource, NSNetServiceBrowserDelegate {
    
    @IBOutlet var tableView: UITableView!
    
    var services: [NSNetService] = [NSNetService]()
    var serviceBrowser: NSNetServiceBrowser? {
        willSet {
            serviceBrowser?.delegate = nil
        }
        
        didSet {
            serviceBrowser?.delegate = self
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        serviceBrowser = NSNetServiceBrowser()
        serviceBrowser?.searchForServicesOfType("_sip._udp", inDomain: "local")
    }
    
    @IBAction func skip(sender: UIBarButtonItem) {
        openUserAccount(nil)
    }
    
    func openUserAccount(gateway: Gateway?) {
        let vc = UIStoryboard(name: "Main", bundle: nil).instantiateViewControllerWithIdentifier("UserAccountViewController") as! UserAccountViewController
        vc.gateway = gateway
        navigationController?.pushViewController(vc, animated: true)
    }
    
    
    // MARK: - NSNetServiceBrowserDelegate
    
    func netServiceBrowser(browser: NSNetServiceBrowser, didFindDomain domainString: String, moreComing: Bool) {
        
    }
    
    func netServiceBrowser(browser: NSNetServiceBrowser, didFindService service: NSNetService, moreComing: Bool) {
        
        services.append(service)
        tableView.beginUpdates()
        tableView.insertRowsAtIndexPaths([NSIndexPath(forRow: services.count - 1, inSection: 0)], withRowAnimation: .Automatic)
        tableView.endUpdates()
    }
    
    func netServiceBrowser(browser: NSNetServiceBrowser, didNotSearch errorDict: [String : NSNumber]) {
        
    }
    
    func netServiceBrowser(browser: NSNetServiceBrowser, didRemoveDomain domainString: String, moreComing: Bool) {
        
    }
    
    func netServiceBrowser(browser: NSNetServiceBrowser, didRemoveService service: NSNetService, moreComing: Bool) {
        
        if let index = services.indexOf(service) {
            services.removeAtIndex(index)
            tableView.beginUpdates()
            tableView.deleteRowsAtIndexPaths([ NSIndexPath(forRow: index, inSection: 0)], withRowAnimation: .Automatic)
            tableView.endUpdates()
        }
        
    }
    
    func netServiceBrowserDidStopSearch(browser: NSNetServiceBrowser) {
        serviceBrowser = nil
    }
    
    func netServiceBrowserWillSearch(browser: NSNetServiceBrowser) {
        
    }
    
    // MARK: - UITableViewDataSource
    
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return services.count
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("GatewayCell", forIndexPath: indexPath)
        
        let svc = services[indexPath.row]
        cell.textLabel?.text = svc.name
        
        return cell
    }
    
    // MARK: - UITableViewDelegate
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        tableView.deselectRowAtIndexPath(indexPath, animated: true)
        
        let svc = services[indexPath.row]
        let gateway = Gateway(netService: svc)
        gateway.resolve()
        
        openUserAccount(gateway)
    }
    
}
