//
//  UIViewController+Call.swift
//  VMS
//
//  Copyright © 2015 Skymotion. All rights reserved.
//

import UIKit
import AddressBook
import Contacts

extension UIViewController {
    func startCall(number: String, displayType: String?, displayName: String?, completion: (Void) -> Void) {
 
        let pho = Phone.sharedInstance
        if pho.status != Phone.RegistrationStatus.Registered {
            let alert = UIAlertView(title: NSLocalizedString("phone.registration.error.title", comment: ""),
                message: NSLocalizedString("phone.registration.error.message", comment: ""),
                delegate: nil,
                cancelButtonTitle: NSLocalizedString("ok", comment: ""))
            alert.show()
            return
        }
        
        let set = NSCharacterSet(charactersInString: "/() -.,")
        let comps = number.componentsSeparatedByCharactersInSet(set)
        let num = (comps as NSArray).componentsJoinedByString("")
        if let domain = pho.getServer() {
            let url = "sip:\(num)@\(domain)"
            let sb = UIStoryboard(name: "Call", bundle: nil)
            let vc: CallViewController = sb.instantiateInitialViewController() as! CallViewController
            vc.nameOrPhone = displayName
            vc.numberType = displayType
            vc.number = url
            presentViewController(vc, animated: false, completion: completion)
        }
        
    }
    
    func startCall(number: String, displayType: String?, displayName: String?) {
        startCall(number, displayType: displayType, displayName: displayName, completion: {})
    }
    
    func callPerson(person: ABRecord, property: ABPropertyID, identifier: ABMultiValueIdentifier) -> Bool {
        if property == kABPersonPhoneProperty {
            if let mv = ABRecordCopyValue(person, property) {
                let multiValue: ABMultiValue = mv.takeRetainedValue() as ABMultiValue
                let index = ABMultiValueGetIndexForIdentifier(multiValue, identifier)
                let value = ABMultiValueCopyValueAtIndex(multiValue, index)
                let valueString: AnyObject = value.takeRetainedValue()
                DDLogVerbose("Calling \(valueString)")
                if let number = valueString as? String {
                    let name = ContactCache.displayFullName(person)
                    let label = ABMultiValueCopyLabelAtIndex(multiValue, index).takeRetainedValue()
                    let displayLabel = ABAddressBookCopyLocalizedLabel(label).takeRetainedValue() as String
                    startCall(number, displayType: displayLabel, displayName: name)
                }
            }
            
            return false
        } else {
            return true
        }
    }
    
    @available(iOS 9.0, *)
    func callContact(contact: CNContact, property: CNContactProperty) -> Bool {
        if property.key == CNContactPhoneNumbersKey {
            let pnum = property.value as! CNPhoneNumber
            let number = pnum.stringValue
            let name = ContactCache.displayFullName9(contact)
            let label = property.label
            let displayLabel: String
            if let lab = label {
                displayLabel = CNContact.localizedStringForKey(lab)
            } else {
                displayLabel = NSLocalizedString("phone.number.unknown", comment: "")
            }
            startCall(number, displayType: displayLabel, displayName: name)
            return false
        } else {
            return true
        }
        
    }
}
