//
//  Call+Utils.swift
//  VMS
//
//  Copyright (c) 2015 Skymotion. All rights reserved.
//

import CoreData

private var todayFormat : NSDateFormatter? = nil
private var dayFormat : NSDateFormatter? = nil
private let durationFormatter = DurationFormatter()

extension Call {
    
    var missed: Bool {
        get {
            return incoming.boolValue && duration.integerValue == 0
        }
    }
    
    func localizedDate() -> String {
        let cal = NSCalendar.currentCalendar()
        let now = NSDate()
        let nowComponents = cal.components([.Day, .Month, .Year], fromDate: now)
        let todayMidnight = cal.dateFromComponents(nowComponents)!
        let yesterdayMidnight = cal.dateByAddingUnit(.Day, value: -1, toDate: todayMidnight, options: [])!
        let oneWeekAgo = cal.dateByAddingUnit(.Day, value: -7, toDate: todayMidnight, options: [])!
        
        if date.compare(todayMidnight) == .OrderedDescending {
            return displayTime()
        } else if date.compare(yesterdayMidnight) == .OrderedDescending {
            return NSLocalizedString("date.yesterday", comment: "")
        } else if date.compare(oneWeekAgo) == .OrderedDescending {
            let day = cal.component(NSCalendarUnit.Weekday, fromDate: date)
            return cal.weekdaySymbols[day - 1]
        } else {
            Call.setupDayFormat()
            return dayFormat!.stringFromDate(date)
        }
    }
    
    func displayTime() -> String {
        Call.setupTodayFormat()
        return todayFormat!.stringFromDate(date)
    }
    
    func displayDate() -> String {
        let cal = NSCalendar.currentCalendar()
        let now = NSDate()
        let nowComponents = cal.components([.Day, .Month, .Year], fromDate: now)
        let todayMidnight = cal.dateFromComponents(nowComponents)!
        let yesterdayMidnight = cal.dateByAddingUnit(.Day, value: -1, toDate: todayMidnight, options: [])!
        
        if date.compare(todayMidnight) == .OrderedDescending {
            return NSLocalizedString("date.today", comment: "")
        } else if date.compare(yesterdayMidnight) == .OrderedDescending {
            return NSLocalizedString("date.yesterday", comment: "")
        } else {
            Call.setupDayFormat()
            return dayFormat!.stringFromDate(date)
        }
    }
    
    func displayNumber() -> String? {
        return Phone.sharedInstance.phone.sipUriUsername(number)
    }

    func displayName() -> String? {
        var displayName: String? = nil
        if let mtch = match {
            if let contact = mtch.contact {
                displayName = contact.displayFullName()
            }
        }
        if displayName == nil {
            displayName = displayNumber()
        }
        return displayName
    }

    func displayLabel() -> String {
        if let mtch = match {
            return mtch.displayLabel()
        }
        
        return NSLocalizedString("phone.number.unknown", comment: "")
    }
    
    func displayCallType() -> String {
        let key: String
        if incoming.boolValue {
            if duration.integerValue == 0 {
                key = "call.state.missed"
            } else {
                key = "call.state.incoming"
            }
        } else {
            if duration.integerValue == 0 {
                key = "call.state.cancelled"
            } else {
                key = "call.state.outgoing"
            }
        }
        
        return NSLocalizedString(key, comment: "")
    }
    
    func displayDuration() -> String? {
        return durationFormatter.stringFromNumber(duration)
    }

    class func setupTodayFormat() {
        if todayFormat == nil {
            todayFormat = NSDateFormatter()
            todayFormat?.dateStyle = .NoStyle
            todayFormat?.timeStyle = .ShortStyle
        }
    }
    
    class func setupDayFormat() {
        if dayFormat == nil {
            dayFormat = NSDateFormatter()
            dayFormat?.dateStyle = .MediumStyle
            dayFormat?.timeStyle = .NoStyle
        }
    }
}
