//
//  ContactCache+UI.swift
//  VMS
//
//  Copyright (c) 2015 Skymotion. All rights reserved.
//

import UIKit
import AddressBook
import Contacts

extension ContactCache {
    func displayFullName(font: UIFont) -> NSAttributedString {
        
        fillFormatInfo()
        
        var first: String? = nil
        var last: String? = nil
        
        let indexFirst = indexFirstname.boolValue
        if indexFirst {
            first = indexedName
            last = lastname
        } else {
            first = firstname
            last = indexedName
        }
        
        let name: NSMutableAttributedString = NSMutableAttributedString()
        let attr = [ NSFontAttributeName: font ]
        
        var delimiter: NSAttributedString
        if let del = compositeNameDelimiter {
            delimiter = NSAttributedString(string: del)
        } else {
            delimiter = NSAttributedString(string: " ")
        }
        
        var attrFirst: NSAttributedString? = nil
        var attrLast: NSAttributedString? = nil
        
        if let f = first {
            if indexFirst {
                attrFirst = NSAttributedString(string: f, attributes: attr)
            } else {
                attrFirst = NSAttributedString(string: f)
            }
        }
        if let l = last {
            if indexFirst {
                attrLast = NSAttributedString(string: l)
            } else {
                attrLast = NSAttributedString(string: l, attributes: attr)
            }
        }
        
        var components = [NSAttributedString]()
        
        if #available(iOS 9.0, *) {
            var cFormat = CNContactDisplayNameOrder.GivenNameFirst
            if let fmt = compositeNameFormat {
                if let cfor = CNContactDisplayNameOrder(rawValue: fmt.integerValue) {
                    cFormat = cfor
                }
            }
            
            if cFormat == .GivenNameFirst {
                if let f = attrFirst {
                    components.append(f)
                }
                if let l = attrLast {
                    components.append(l)
                }
            } else {
                if let l = attrLast {
                    components.append(l)
                }
                if let f = attrFirst {
                    components.append(f)
                }
            }
        } else {
            var cFormat = kABPersonCompositeNameFormatFirstNameFirst
            if let fmt = compositeNameFormat {
                cFormat = fmt.integerValue
            }
            
            if cFormat == kABPersonCompositeNameFormatFirstNameFirst {
                if let f = attrFirst {
                    components.append(f)
                }
                if let l = attrLast {
                    components.append(l)
                }
            } else {
                if let l = attrLast {
                    components.append(l)
                }
                if let f = attrFirst {
                    components.append(f)
                }
            }
        }
        
        let len = components.count
        for var i = 0; i < len; i++ {
            if i > 0 {
                name.appendAttributedString(delimiter)
            }
            name.appendAttributedString(components[i])
        }
        
        return name
    }
    
    func displayFullName() -> String {
        fillFormatInfo()
        
        var first: String? = nil
        var last: String? = nil
        
        let indexFirst = indexFirstname.boolValue
        if indexFirst {
            first = indexedName
            last = lastname
        } else {
            first = firstname
            last = indexedName
        }
        
        if #available(iOS 9.0, *) {
            var cFormat = CNContactDisplayNameOrder.GivenNameFirst
            if let fmt = compositeNameFormat {
                if let cfor = CNContactDisplayNameOrder(rawValue: fmt.integerValue) {
                    cFormat = cfor
                }
            }
            var delimiter = " "
            if let del = compositeNameDelimiter {
                delimiter = del
            }
            return ContactCache.displayFullName(first, last: last, nameOrder: cFormat, delimiter: delimiter)
        } else {
            var cFormat = UInt32(kABPersonCompositeNameFormatFirstNameFirst)
            if let fmt = compositeNameFormat {
                cFormat = UInt32(fmt.integerValue)
            }
            var delimiter = " "
            if let del = compositeNameDelimiter {
                delimiter = del
            }
            return ContactCache.displayFullName(first, last: last, nameFormat: cFormat, delimiter: delimiter)
        }
    }
    
    private func fillFormatInfo() {
        if compositeNameFormat == nil {
            var nameFormat: NSNumber? = nil
            var nameDelimiter: String? = nil
            if #available(iOS 9, *) {
                let store = CNContactStore()
                do {
                    let person = try store.unifiedContactWithIdentifier(self.identifier, keysToFetch: [CNContactGivenNameKey, CNContactFamilyNameKey])
                    let (format, delimiter) = ContactCache.nameFormatForContact(person)
                    nameFormat = NSNumber(integer: format.rawValue)
                    nameDelimiter = delimiter
                } catch let err as NSError {
                    DDLogError("Error fetching contact: \(err) \(err.userInfo)")
                }
            } else {
                let rhab = AddressBook.sharedInstance.addressBook!
                let rid = self.recordID.intValue
                rhab.performAddressBookAction({ (addressBook: ABAddressBook!) in
                    let record: ABRecord = ABAddressBookGetPersonWithRecordID(addressBook, rid).takeRetainedValue()
                    let (format, delimiter) = ContactCache.nameFormatForRecord(record)
                    
                    nameFormat = NSNumber(int: Int32(format.value))
                    nameDelimiter = delimiter as String
                }, waitUntilDone: true)
            }
            self.compositeNameFormat = nameFormat
            self.compositeNameDelimiter = nameDelimiter
        }
    }
    
    // MARK: iOS 8
    
    private class func nameFormatForRecord(record: ABRecord) -> (ABPersonCompositeNameFormat, String) {
        let format = ABPersonGetCompositeNameFormatForRecord(record)
        let delimiter = ABPersonCopyCompositeNameDelimiterForRecord(record).takeUnretainedValue()
        
        return (format, delimiter as String)
    }
    
    private class func displayFullName(first: String?, last: String?, nameFormat: ABPersonCompositeNameFormat, delimiter: String) -> String {
        let cFormat = Int(nameFormat)
        
        var components = [String]()
        if cFormat == kABPersonCompositeNameFormatFirstNameFirst {
            if let f = first {
                components.append(f)
            }
            if let l = last {
                components.append(l)
            }
        } else {
            if let l = last {
                components.append(l)
            }
            if let f = first {
                components.append(f)
            }
        }
        
        return (components as NSArray).componentsJoinedByString(delimiter)
    }
    
    private class func displayFullName(record: ABRecord, nameFormat: ABPersonCompositeNameFormat, delimiter: String) -> String {
        var first: String?
        if let fi = ABRecordCopyValue(record, kABPersonFirstNameProperty) {
            first = fi.takeRetainedValue() as? String
        }
        
        var last: String?
        if let la = ABRecordCopyValue(record, kABPersonLastNameProperty) {
            last = la.takeRetainedValue() as? String
        }
        
        return displayFullName(first, last: last, nameFormat: nameFormat, delimiter: delimiter)
    }
    
    class func displayFullName(record: ABRecord) -> String {
        let (format, delimiter) = nameFormatForRecord(record)
        
        return displayFullName(record, nameFormat: format, delimiter:  delimiter)
    }
    
    // MARK: iOS 9
    
    @available(iOS 9.0, *)
    private class func nameFormatForContact(contact: CNContact) -> (CNContactDisplayNameOrder, String) {
        var format = CNContactFormatter.nameOrderForContact(contact)
        if format == .UserDefault {
            format = .GivenNameFirst
        }
        let delimiter = CNContactFormatter.delimiterForContact(contact)
        
        return (format, delimiter as String)
    }
    
    @available(iOS 9.0, *)
    private class func displayFullName(first: String?, last: String?, nameOrder: CNContactDisplayNameOrder, delimiter: String) -> String {
        var cFormat = nameOrder
        
        if cFormat == .UserDefault {
            cFormat = .GivenNameFirst
        }
        
        var components = [String]()
        if cFormat == .GivenNameFirst {
            if let f = first {
                components.append(f)
            }
            if let l = last {
                components.append(l)
            }
        } else {
            if let l = last {
                components.append(l)
            }
            if let f = first {
                components.append(f)
            }
        }
        
        return (components as NSArray).componentsJoinedByString(delimiter)
    }
    
    @available(iOS 9.0, *)
    private class func displayFullName(contact: CNContact, nameOrder: CNContactDisplayNameOrder, delimiter: String) -> String {
        var first: String? = contact.givenName
        if first!.isEmpty {
            first = nil
        }
        
        var last: String? = contact.familyName
        if last!.isEmpty {
            last = nil
        }
        
        return displayFullName(first, last: last, nameOrder: nameOrder, delimiter: delimiter)
    }
    
    @available(iOS 9.0, *)
    class func displayFullName9(contact: CNContact) -> String {
        let (format, delimiter) = nameFormatForContact(contact)
        
        return displayFullName(contact, nameOrder: format, delimiter:  delimiter)
    }
}
