//
//  ExponentialBackOff.swift
//  VMS
//
//  Copyright (c) 2015 Skymotion. All rights reserved.
//

import Foundation

public struct ExponentialBackOff : BackOff {
    
    static let DEFAULT_INITIAL_INTERVAL : NSTimeInterval = 0.5
    static let DEFAULT_MAX_INTERVAL : NSTimeInterval = 60
    static let DEFAULT_MULTIPLIER : Double = 1.5
    
    private let initialInterval : NSTimeInterval
    private let maxInterval : NSTimeInterval
    private let multiplier : Double
    private var currentInterval : NSTimeInterval = 0
    
    public init(initial: NSTimeInterval, max: NSTimeInterval, multiplier: Double) {
        initialInterval = initial
        maxInterval = max
        self.multiplier = multiplier
    }
    
    public init() {
        self.init(initial: ExponentialBackOff.DEFAULT_INITIAL_INTERVAL, max: ExponentialBackOff.DEFAULT_MAX_INTERVAL, multiplier: ExponentialBackOff.DEFAULT_MULTIPLIER)
    }
    
// MARK: BackOff
    
    mutating public func nextBackOffInterval() -> NSTimeInterval {
        if(currentInterval == 0) {
            currentInterval = initialInterval;
        } else if(currentInterval < maxInterval) {
            currentInterval = (multiplier * currentInterval);
            if(currentInterval > maxInterval) {
                currentInterval = maxInterval;
            }
        }
        
        return currentInterval;
    }
    
    mutating public func reset() {
        currentInterval = 0
    }
    
}